/*
  theta.cpp
*/

#include <iostream>
#include <sdpsl.h>

using namespace std;
using namespace sdp;

int main()
{
  SDPVariable X(5);
  SDPConstraintContainer cc;

  /* We first generate one constraint for each edge of the 5-cycle,
     using masks for efficiency. 
  */
  MatrixMask mask(5);
  Matrix<double> J(5, 5, 1);  // A 5 x 5 all-ones matrix
  SDPLinearConstraint edge_constraint = SDPMaskedMatrix(J, mask) * X == 0;

  for (int i = 0; i < 5; i++) {
    int j = (i + 1) % 5;

    mask.turn_on(i, j);
    cc.add_constraint(edge_constraint);
    mask.turn_off(i, j);
  }

  /* The trace == 1 constraint */
  cc.add_constraint(Matrix<double>::identity(5) * X == 1);

  /* We add the constraints to the problem */
  SDPProblem problem;

  problem << cc;

  /* We set the objective function */
  problem.set_objective(J * X);

  /* And we output the SDPA file */
  cout << SDPAWriter(problem);

  return 0;
}
