from sage.sdpsl import *

def fans_theorem(A, k):
    if not A.is_square():
        raise ValueError('A must be a square matrix')
        
    n = A.nrows()

    # Variables and the constraint container
    X, Y = SDPVariable(n), SDPVariable(n)
    cc = SDPConstraintContainer()

    # First the trace constraint
    cc.add_constraint(identity_matrix(RR, n) * X == k)

    # Then the correspondence between X and Y
    E = SDPMatrix(n, 1)
    mask = SDPMatrixMask(n)
    C = (E, mask) * X + (E, mask) * Y == 0

    for i in xrange(n):
        for j in xrange(i, n):
            if i == j: 
                C.set_rhs(1)
            else:
                C.set_rhs(0)

            mask[i, j] = True
            cc.add_constraint(C)
            mask[i, j] = False

    # Then we create the problem and return it
    problem = SDPProblem()

    problem.set_objective(A * X)
    problem.add_container(cc)

    return problem

