from sage.sdpsl import *

def poly_min_sos(p):
    PR = PolynomialRing(RR, 'x')
    x = PR.gen()

    try:
        q = PR(p)
    except:
        raise ValueError('p must be a univariate polynomial')

    d = p.degree()

    if d % 2 != 0:
        raise ValueError('p must have even degree')

    # Variables for the SDP
    X = SDPVariable(1 + d / 2)
    s, t = SDPVariable(1), SDPVariable(1)
    cc = SDPConstraintContainer()
    
    # The constraint
    v = vector(PR, [ x^k for k in xrange(1 + d / 2) ])
    R = v.column() * v.row()  # This is a moment matrix

    cc.add_constraint(R * X + 1 * s + (-1) * t == q)

    # We create the problem and return it
    problem = SDPProblem()

    problem.set_objective(1 * s + (-1) * t)
    problem.add_container(cc)

    return problem

