from sage.sdpsl import *

# Here G is a SAGE graph
def make_theta_sdp(G):
    # Vertices are not necessarily integers; we associate integers with the vertices.
    vidx = dict(zip(G.vertices(), range(G.num_verts())))

    # Constraint generation
    cc = SDPConstraintContainer()
    
    X = SDPVariable(G.num_verts())
    J = SDPMatrix(G.num_verts(), initial_value = 1)

    # Creates the constraints for the edges
    mask = SDPMatrixMask(G.num_verts())
    edge_constraint = (J, mask) * X == 0

    for u, v in G.edges(labels = False):
        mask[vidx[u], vidx[v]] = True
        cc.add_constraint(edge_constraint)
        mask[vidx[u], vidx[v]] = False

    # Then the trace constraint
    cc.add_constraint(identity_matrix(RR, G.num_verts()) * X == 1)

    # And the problem
    problem = SDPProblem()

    problem.set_objective(J * X)
    problem.add_container(cc)

    return problem

