/*!
  \file laguerre.h

  \brief Functions for generating Laguerre polynomials.

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LAGUERRE_H__
#define __LAGUERRE_H__

#include <vector>
#include <sdpsl/polynomial.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else

namespace sdp {
#endif

/*!
  \fn void compute_laguerre_polys(int n, const PolynomialVariable& x, const Number& alpha, 
				   std::vector<Polynomial>& out_list, const Number& c = 1, 
				   int power = 1);

  \brief Generates a list of Laguerre polynomials.
  
  Puts in \a out_list the <a
  href="http://en.wikipedia.org/wiki/Laguerre_polynomials">Laguerre
  Polynomials</a> \f$L_0^\alpha(c x^k), \ldots, L_n^\alpha(c x^k)\f$,
  where \f$\alpha\f$ is the paramter \a alpha given to the function,
  and \f$k\f$ is the parameter \a power.
*/
extern void compute_laguerre_polys(int n, const PolynomialVariable& x, const Number& alpha, 
				   std::vector<Polynomial>& out_list, const Number& c = 1, 
				   int power = 1);

}  // namespace

#endif

// Local variables:
// mode: c++
// End:
