/*!
  \file polyerror.h

  \brief Exceptions for the polynomial classes.

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __POLYERROR_H__
#define __POLYERROR_H__

#include <string>
#include <exception>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

//! Exceptions for the polynomial classes are member classes of PolyError.
class PolyError {
public:

  //! Variable requested is invalid. \sa PolynomialVariable.
  class InvalidVariable : public std::exception {
  public:
    virtual const char *what() const throw() { return "Invalid variable index"; }
  };

  //! Invalid parameter passed to function. \sa Polynomial
  class ValueError : public std::exception {
    std::string what_text;

  public:
    ValueError() { }
    explicit ValueError(const std::string& txt) : what_text(txt) { }

    virtual const char *what() const throw() { return what_text.c_str(); }

 };
};

}  // namespace

#endif

// Local variables:
// mode: c++
// End:
