/*
  laguerre.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/laguerre.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

void compute_laguerre_polys(int n, const PolynomialVariable& x, const Number& alpha, 
			    std::vector<Polynomial>& out_list, const Number& c, 
			    int power)
{
  out_list.clear();
  
  /* The first two polynomials */
  out_list.push_back(Polynomial(1));
  out_list.push_back(Number(-c) * x(power) + alpha + 1);

  /* The other ones */
  for (int k = 2; k <= n; k++) {
    out_list.push_back((2 + (Number(alpha - 1) - c * x(power)) / k) * out_list[k - 1]);
    out_list[k] -= Number(1 + (alpha - 1) / k) * out_list[k - 2];
  }
}

}  // namespace
