/*
  matrixmask.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/matrixmask.h>
#include <climits>

using namespace std;

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

/*
  Constructors
*/
MatrixMask::MatrixMask(int nn)
  : m(nn), n(nn), data(std::make_shared<Data>(1 + (nn * nn) / CHAR_BIT))
{

}

MatrixMask::MatrixMask(int mm, int nn)
  : m(mm), n(nn), data(std::make_shared<Data>(1 + (mm * nn) / CHAR_BIT))
{

}

MatrixMask::MatrixMask(const MatrixMask& other)
  : m(other.m), n(other.n), 
    data(std::make_shared<Data>(*other.data))
{

}

/*
  Other operations
*/
bool MatrixMask::is_on(int i, int j) const
{
  int pos = i * n + j;
  int byte = pos / CHAR_BIT;
  int bit = pos % CHAR_BIT;
  
  return data->mask[byte] & (1 << bit);
}

void MatrixMask::turn_on(int i, int j)
{
  int pos = i * n + j;
  int byte = pos / CHAR_BIT;
  int bit = pos % CHAR_BIT;

  if (data->mask[byte] & (1 << bit)) return;

  data->mask[byte] = data->mask[byte] | (1 << bit);
  data->on_pos.push_back(Position(i, j));
}

void MatrixMask::turn_off(int i, int j)
{
  int pos = i * n + j;
  int byte = pos / CHAR_BIT;
  int bit = pos % CHAR_BIT;

  if (!(data->mask[byte] & (1 << bit))) return;

  data->mask[byte] = data->mask[byte] ^ (1 << bit);
  data->on_pos.remove(Position(i, j));
}

void MatrixMask::clear()
{
  for (auto& p : data->on_pos) {
    int pos = p.i * n + p.j;
    int byte = pos / CHAR_BIT;

    data->mask[byte] = 0;
  }

  data->on_pos.clear();
}

}  // namespace
