/*
  number.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/number.h>
#include <sstream>
#include <cstdlib>
#include <exception>
#include <ctype.h>
#include <sdpsl/sdperror.h>

using namespace std;

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

/*
  operator<< for InsideCoefficient
*/
ostream& operator<<(ostream& out, const InsideCoefficient& ic)
{
  bool positive = ic.num >= 0 ? true : false;
  Number abs = positive ? ic.num : Number(-ic.num);

  if (positive)
    out << "+ ";
  else
    out << "- ";

  out << abs;

  return out;  
}

/*
  from_string
*/
void NumberOperations::from_string(const std::string& str, double& out)
{
  char *endptr;
  out = strtod(str.c_str(), &endptr);

  if (endptr && *endptr)
    throw ValueError(ERROR_MSG("Error converting number"));
}

void NumberOperations::from_string(const std::string& str, mpf_class& out)
{
  try {
    out = mpf_class(str);
  }
  catch (std::invalid_argument) {
    throw ValueError(ERROR_MSG("Error converting number"));
  }
}

}  // namespace
