/*
  rawconstraint.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/rawconstraint.h>

using namespace std;

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

void MatrixEntryList::add_matrix_entries(double output_eps, int var_idx, 
					 const SDPMaskedMatrix& mm)
{
  const Matrix<Number>& A = mm.matrix();
  const MatrixMask *mask = mm.mask();

  if (!mask)
    for (int i = 0; i < A.num_rows(); i++)
      for (int j = i; j < A.num_cols(); j++)
	add_entry(output_eps, var_idx, i, j, (A[i][j] + A[j][i]) / 2);
  else
    for (auto& pos : *mask) {
      int i = pos.i, j = pos.j;

      if (i <= j && mask->is_on(j, i))
	add_entry(output_eps, var_idx, i, j, (A[i][j] + A[j][i]) / 2);
      else if (i <= j)
	add_entry(output_eps, var_idx, i, j, A[i][j] / 2);
      else if (!mask->is_on(j, i))  // Here, i > j
	add_entry(output_eps, var_idx, j, i, A[i][j] / 2);
    }	
}

}  // namespace
