/*
  sdplin.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/sdplin.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

/*
  Constructors
*/
SDPLinearCombination::SDPLinearCombination(const SDPVariable& X, const SDPMaskedMatrix& mm)
{
  if (mm.matrix().num_rows() != mm.matrix().num_cols())
    throw ValueError(ERROR_MSG("Matrix is not square"));

  if (mm.matrix().num_rows() != X.size())
    throw ValueError(ERROR_MSG("Matrix and variable have different sizes"));

  summands.insert(SummandMap::value_type(X, mm));
}

/*
  Operators
*/
SDPLinearCombination& SDPLinearCombination::operator+=(const SDPLinearCombination& other)
{
  for (auto& s : other.summands) {
    auto ret = summands.insert(s);

    if (!ret.second)
      throw ValueError(ERROR_MSG("Linear combinations have common variable"));
  }

  return *this;
}

SDPLinearCombination operator*(const SDPMaskedMatrix& mat, const SDPVariable& X)
{
  return SDPLinearCombination(X, mat);
}

SDPLinearCombination operator+(const SDPLinearCombination& a, const SDPLinearCombination& b)
{
  SDPLinearCombination ret = a;
  ret += b;

  return ret;
}

}  // namespace
