/*
  sdpproblem.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <set>
#include <sdpsl/sdpproblem.h>

using namespace std;

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

void SDPProblem::set_objective(const SDPLinearCombination& C)
{
  _obj_variables.clear();
  _objective_entries.clear();

  for (auto& s : C) {
    _obj_variables.push_back(s.first);
    _objective_entries.add_matrix_entries(env.output_eps(), s.first.idx(), s.second);
  }
}

std::list<SDPVariable> SDPProblem::variables() const
{
  set<SDPVariable> var_set;
  
  for (auto& var : _obj_variables)
    var_set.insert(var);
  
  for (auto& cc : cc_list)
    for (auto& var : cc->variables())
      var_set.insert(var);

  list<SDPVariable> ret;

  for (auto& var : var_set)
    ret.push_back(var);

  return ret;
}

SDPProblem& SDPProblem::operator<<(const SDPConstraintContainer& cc)
{
  cc_list.push_back(const_shared_ref<SDPConstraintContainer>(cc));

  return *this;
}

}  // namespace

