/*
  tensortemp.cpp

  Copyright (c) 2014 Fernando M. de Oliveira Filho
  <http://www.ime.usp.br/~fmario>
  
  This file is part of SDPSL.
  
  SDPSL is free software: you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free
  Software Foundation, either version 3 of the License, or (at your
  option) any later version.
  
  SDPSL is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
  for more details.
  
  You should have received a copy of the GNU General Public License
  along with SDPSL.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <sdpsl/tensortemp.h>

#ifdef __MPF_BUILD__
namespace mpf_sdp {
#else
namespace sdp {
#endif

/*
  Constructs a TensorIter given a list of DimBound objects.  
*/
TensorTemplate::TensorTemplate(const std::initializer_list<DimBounds>& bounds)
  : num_dimensions(bounds.size()), lower(bounds.size()), upper(bounds.size()),
    size(bounds.size()), displace(bounds.size())
{
  int i = 0;
  for (const DimBounds& db : bounds) {
    lower[i] = db.lower;
    upper[i] = db.upper;
    size[i] = db.upper - db.lower + 1;
    i++;
  }

  int cumul = 1;
  for (i = num_dimensions - 1; i >= 0; i--) {
    displace[i] = cumul;
    cumul *= size[i];
  }

  mat_size = cumul;
}

/*
  idx

  TODO: better description
*/
int TensorTemplate::idx(const int *list) const
{
  int idx = 0;
  for (int i = 0; i < num_dimensions; i++)
    idx += (list[i] - lower[i]) * displace[i];
  
  return idx;
}

//
// Iterator
//

/*
  Constructs an iterator pointing at the beginning.
*/
TensorTemplate::iterator::iterator(const TensorTemplate& temp, int cidx)
  : mat_size(temp.matrix_size()), num_dimensions(temp.num_dimensions), lower(temp.lower),
    upper(temp.upper), cur(temp.num_dimensions, 0), cur_idx(cidx)
{
  for (int i = 0; i < num_dimensions; i++)
    cur[i] = lower[i];
}

}  // namespace
